#include <stdio.h>
#include <stdlib.h>
#include <gmp.h>
#include <math.h>

struct trans_node
{
    mpz_t a;
    mpz_t b;
    mpz_t c;
    mpz_t d;
};

typedef struct trans_node TRANSNODE;

struct ans_node
{
    mpq_t ans;
    struct ans_node * next;
};

typedef struct ans_node ANSNODE;

void restore_ans(TRANSNODE* trans_node, mpq_t ans)
{
    mpq_t num, den, temp;
    mpq_init(num);
    mpq_init(den);
    mpq_init(temp);
    mpq_set_z(temp, trans_node->a);
    mpq_mul(num, ans, temp);
    mpq_set_z(temp, trans_node->b);
    mpq_add(num, num, temp);
    mpq_set_z(temp, trans_node->c);
    mpq_mul(den, ans, temp);
    mpq_set_z(temp, trans_node->d);
    mpq_add(den, den, temp);
    mpq_div(ans, num, den);
    mpq_clear(num);
    mpq_clear(den);
    mpq_clear(temp);
}

TRANSNODE * init_trans_node()
{
    TRANSNODE * ret = (TRANSNODE * )malloc(sizeof(TRANSNODE));
    mpz_init(ret->a);
    mpz_init(ret->b);
    mpz_init(ret->c);
    mpz_init(ret->d);
    return ret;
}

void free_trans_node(TRANSNODE * node)
{
    mpz_clear(node->a);
    mpz_clear(node->b);
    mpz_clear(node->c);
    mpz_clear(node->d);
    free(node);
}

ANSNODE * init_ans_node()
{
    ANSNODE * ret = (ANSNODE * )malloc(sizeof(ANSNODE));
    mpq_init(ret->ans);
    ret->next = NULL;
    return ret;
}

void free_ans_node(ANSNODE * node)
{
    mpq_clear(node->ans);
    free(node);
}

ANSNODE* get_ans(ANSNODE * ans_node, TRANSNODE* trans_node, int level)
{
    if(level == 1)
    {
        ANSNODE * new_ans = init_ans_node();
        mpq_set_si(new_ans->ans, 1, 2);
        restore_ans(trans_node, new_ans->ans);
        ans_node->next = new_ans;
        ans_node = new_ans;
    }
    else
    {
        TRANSNODE * left, * right;
        left = init_trans_node();
        right = init_trans_node();
        mpz_set(left->a, trans_node->b);
        mpz_add(left->b, trans_node->a, trans_node->b);
        mpz_set(left->c, trans_node->d);
        mpz_add(left->d, trans_node->c, trans_node->d);
        mpz_set(right->a, trans_node->a);
        mpz_add(right->b, trans_node->a, trans_node->b);
        mpz_set(right->c, trans_node->c);
        mpz_add(right->d, trans_node->c, trans_node->d);
        ans_node = get_ans(ans_node, left, level - 1);
        ans_node = get_ans(ans_node, right, level - 1);
    }
    return ans_node;
}

void append_neg_ans(ANSNODE * ans_node)
{
    ANSNODE * neg_copy = init_ans_node(), * neg_current, * current, * last;
    ANSNODE * temp = init_ans_node();
    neg_current = neg_copy;
    last = NULL;
    current = ans_node;
    while(current != NULL)
    {
        mpq_neg(temp->ans, current->ans);
        neg_current->next = temp;
        neg_current = temp;
        last = current;
        current = current->next;
        temp = init_ans_node();
    }
    last->next = neg_copy->next;
}

int main(int argc, char* argv[])
{
    int level, i, j, n;
    mpz_t * poly, temp, temp2;
    FILE * file;
    TRANSNODE *trans_node;
    ANSNODE *first_ans;
    
    if(argc != 3)
    {
        printf("Usage branch_poly_gen.exe <level> <out>\n");
        return 0;
    }
    
    file = fopen(argv[2], "w");
    level = atoi(argv[1]);
    n = (int)pow(2, level - 1);
    if(level <= 0)
    {
        printf("Invalid level(must be >= 1)");
        return 0;
    }
    
    trans_node = init_trans_node();
    first_ans = init_ans_node();
    mpz_set_si(trans_node->a, 1);
    mpz_set_si(trans_node->b, 0);
    mpz_set_si(trans_node->c, 0);
    mpz_set_si(trans_node->d, 1);
    get_ans(first_ans, trans_node, level);
    first_ans = first_ans->next;
    append_neg_ans(first_ans);
    poly = (mpz_t * )malloc(sizeof(mpz_t) * (2 * n + 1));
    mpz_init(temp);
    mpz_init(temp2);
    for(i = 0;i < 2 * n + 1;i++)
    {
        mpz_init(poly[i]);
        mpz_set_si(poly[i], 0);
    }
    mpq_out_str(stdout, 10, first_ans->ans);
    printf("\n");
    mpz_set(poly[1], mpq_denref(first_ans->ans));
    mpz_set(poly[0], mpq_numref(first_ans->ans));
    mpz_neg(poly[0], poly[0]);
    first_ans = first_ans->next;
    for(i = 2;i <= 2 * n;i++)
    {
        mpz_set(temp, mpq_denref(first_ans->ans));
        mpz_mul(poly[i], temp, poly[i - 1]);
        for(j = i - 1;j >= 1;j--)
        {
            mpz_set(temp, mpq_numref(first_ans->ans));
            mpz_neg(temp, temp);
            mpz_mul(poly[j], poly[j], temp);
            mpz_set(temp, mpq_denref(first_ans->ans));
            mpz_mul(temp, temp, poly[j - 1]);
            mpz_add(poly[j], poly[j], temp);
        }
        mpz_set(temp, mpq_numref(first_ans->ans));
        mpz_neg(temp, temp);
        mpz_mul(poly[0], temp, poly[0]);
        mpq_out_str(stdout, 10, first_ans->ans);
        printf("\n");
        first_ans = first_ans->next;
    }
    
    for(i = 2 * n;i >= 0;i--)
    {
        fprintf(file, "(");
        mpz_out_str(file, 10, poly[i]);
        if(i == 0)
        {
            fprintf(file, ")");
        }else{
            fprintf(file, ")*x^%d+", i);
        }
    }
    
    fclose(file);
    return 0;
}
