#include <stdio.h>
#include <stdlib.h>
#include <gmp.h>

mpz_t ** poly_cache;

mpz_t * generate(int n)
{
    mpz_t * ret;
    mpz_t * pre_poly1;
    mpz_t * pre_poly2;
    mpz_t temp;
    int i;
    if(poly_cache[n] != NULL)
    {
        return poly_cache[n];
    }
    ret = malloc(sizeof(mpz_t) * (n + 1));
    for(i = 0;i <= n;i++)
    {
        mpz_init(ret[i]);
    }
    if(n == 0)
    {
        mpz_set_si(ret[0], 1);
    }
    if(n == 1)
    {
        mpz_set_si(ret[0], 0);
        mpz_set_si(ret[1], 1);
    }
    if(n <= 1)
    {
        poly_cache[n] = ret;
        return ret;
    }
    pre_poly1 = generate(n - 1);
    pre_poly2 = generate(n - 2);
    
    mpz_init(temp);
    for(i = 0;i <= n - 1;i++)
    {
        mpz_mul_si(ret[i + 1], pre_poly1[i], 2);
    }
    for(i = 0;i <= n - 2;i++)
    {
        mpz_set(temp, pre_poly2[i]);
        mpz_neg(temp, temp);
        mpz_add(ret[i], ret[i], temp);
    }
    mpz_clear(temp);
    for(i = 0;i <= n - 2;i++)
    {
        mpz_clear(pre_poly2[i]);
    }
    free(pre_poly2);
    poly_cache[n] = ret;
    return ret;
}

int main(int argc, char * argv[])
{
    int i, n;
    mpz_t * random_poly;
    FILE * file;
    
    if(argc != 3)
    {
        printf("Usage chebyshev_poly_gen.exe <dim> <out>\n");
        return 0;
    }
    
    file = fopen(argv[2], "w");
    
    n = atoi(argv[1]);
    poly_cache = (mpz_t**)malloc(sizeof(mpz_t * )*(n + 1));
    for(i = 0;i <= n;i++)
    {
        poly_cache[i] = NULL;
    }
    random_poly = generate(n);
    for(i = n;i >= 0;i--)
    {
        fprintf(file, "(");
        mpz_out_str(file, 10, random_poly[i]);
        if(i == 0)
        {
            fprintf(file, ")");
        }else{
            fprintf(file, ")*x^%d+", i);
        }
    }
    
    fclose(file);
    
    return 0;
}
