#include <stdio.h>
#include <stdlib.h>
#include <gmp.h>

mpz_t * pow_cache;

__mpz_struct* get_pow(int n)
{
    if(mpz_cmp_si(pow_cache[n], 0) == 0)
    {
        if(n == 0 || n == 1)
        {
            mpz_set_si(pow_cache[n], 1);
        }
        else
        {
            mpz_set_si(pow_cache[n], n);
            mpz_mul(pow_cache[n], pow_cache[n], get_pow(n - 1));
        }
    }
    return pow_cache[n];
}

mpz_t * generate(int n)
{
    mpq_t * coeffs;
    mpz_t * ret;
    mpq_t temp1, temp2;
    int i;
    coeffs = (mpq_t * )malloc(sizeof(mpq_t) * (n + 1));
    ret = (mpz_t * )malloc(sizeof(mpq_t) * (n + 1));
    for(i = 0;i <= n;i++)
    {
        mpq_init(coeffs[i]);
        mpz_init(ret[i]);
    }
    mpq_init(temp1);
    mpq_init(temp2);
    for(i = n;i >= 0;i--)
    {
        mpq_set_si(coeffs[i], 1, 1);
        mpq_set_z(temp1, get_pow(n));
        mpq_set_z(temp2, get_pow(n - i));
        mpq_div(temp1, temp1, temp2);
        mpq_set_z(temp2, get_pow(i));
        mpq_div(coeffs[i], temp1, temp2);
        mpq_inv(temp1, temp2);
        mpq_mul(coeffs[i], coeffs[i], temp1);
        if(i % 2 == 1)
        {
            mpq_neg(coeffs[i], coeffs[i]);
        }
    }
    mpq_set_z(temp1, mpq_denref(coeffs[n]));
    for(i = n;i >= 0;i--)
    {
        mpq_mul(coeffs[i], coeffs[i], temp1);
        mpq_get_num(ret[i], coeffs[i]);
    }
    mpq_clear(temp1);
    mpq_clear(temp2);
    for(i = 0;i <= n;i++)
    {
        mpq_clear(coeffs[i]);
    }
    free(coeffs);
    return ret;
}

int main(int argc, char * argv[])
{
    int i, n;
    mpz_t * random_poly;
    FILE * file;
    
    if(argc != 3)
    {
        printf("Usage laguerre_poly_gen.exe <dim> <out>\n");
        return 0;
    }
    
    file = fopen(argv[2], "w");
    
    n = atoi(argv[1]);
    pow_cache = (mpz_t * )malloc(sizeof(mpz_t) * (n + 1));
    for(i = 0;i <= n;i++)
    {
        mpz_init(pow_cache[i]);
    }
    random_poly = generate(n);
    for(i = n;i >= 0;i--)
    {
        fprintf(file, "(");
        mpz_out_str(file, 10, random_poly[i]);
        if(i == 0)
        {
            fprintf(file, ")");
        }else{
            fprintf(file, ")*x^%d+", i);
        }
    }
    
    fclose(file);
    
    return 0;
}
