#include <gmp.h>

int lowerboundNPre(mpz_t minusPoly0, mpz_t firstMinusPoly, mpz_t newtonLowerBound, int *sgnv)
{
    mpz_fdiv_q(newtonLowerBound, minusPoly0, firstMinusPoly);
    return sgnv[0] == 1;
}

int lowerboundN(int n,mpz_t *poly, mpz_t lowerBound, mpz_t newtonCandidate, int polyZeroSgn){

  int j,k;
  mpz_t tmp3, tmp4;

  mm_mpz_get(tmp3);
  mm_mpz_get(tmp4);
  
  // tmp6 is newton lowerbound candidate(r)
  // tmp1 is G_4,G_2's lowerbound, so if they are the same I don't calculate.
  if(mpz_cmp(newtonCandidate, lowerBound) <= 0 || mpz_cmp_si(newtonCandidate, alpha0) <= 0){
    mm_mpz_release(tmp3);
    mm_mpz_release(tmp4);
    return 0;
  }
  
  // tmp3 is a[0], tmp4 is for r^n
  mpz_set(tmp3,poly[0]);
  mpz_set(tmp4, newtonCandidate);
  mpz_addmul(tmp3,poly[1],tmp4);

  for(j=2;j<=n-1;j++){
    
    mpz_mul(tmp4, tmp4, newtonCandidate);
    mpz_addmul(tmp3,poly[j],tmp4);
    
    // check sign
    if((k=mpz_sgn(tmp3)) != polyZeroSgn && k!=0)
    {
        mm_mpz_release(tmp3);
        mm_mpz_release(tmp4);
        return 0;
    }
    
  }
  
  if(2<=n){

    mpz_mul(tmp4, tmp4, newtonCandidate);
    mpz_addmul(tmp3,poly[n],tmp4);
    
    // check sign
    if(mpz_sgn(tmp3) != polyZeroSgn)
    {
        mm_mpz_release(tmp3);
        mm_mpz_release(tmp4);
        return 0;
    }

  }


  // tmp6(r) is lowerbound
  mpz_set(lowerBound, newtonCandidate);
  
#ifdef PRINTLB
  gmp_printf("<N:%Zd>", lowerBound);
#endif

  mm_mpz_release(tmp3);
  mm_mpz_release(tmp4);
  
  return 1;
}
