// Remarks
// This log ignore sign, so you can get log of negative number.


#ifndef MPZ_LOG
#define MPZ_LOG
#include <stdio.h>
#include <stdlib.h>
#ifndef __GMP_H__
#include <gmp.h>
#endif

//#define BIT32
#ifdef BIT32
#define MSB(x) MSB32bit((x))
#else
#define MSB(x) MSB64bit((x))
#endif

int count32bit(unsigned int v) {
    unsigned count = (v & 0x55555555) + ((v >> 1) & 0x55555555);
    count = (count & 0x33333333) + ((count >> 2) & 0x33333333);
    count = (count & 0x0f0f0f0f) + ((count >> 4) & 0x0f0f0f0f);
    count = (count & 0x00ff00ff) + ((count >> 8) & 0x00ff00ff);
    return (count & 0x0000ffff) + ((count >> 16) & 0x0000ffff);
}

int MSB32bit(unsigned int v) {
    if (v == 0)
    {
        printf("no bit.\n");
        abort();
    }
    v |= (v >> 1);
    v |= (v >> 2);
    v |= (v >> 4);
    v |= (v >> 8);
    v |= (v >> 16);
    return count32bit(v) - 1;
}

int count64bit(unsigned long long int v) {
    unsigned long long int count = (v & 0x5555555555555555) + ((v >> 1) & 0x5555555555555555);
    count = (count & 0x3333333333333333) + ((count >> 2) & 0x3333333333333333);
    count = (count & 0x0f0f0f0f0f0f0f0f) + ((count >> 4) & 0x0f0f0f0f0f0f0f0f);
    count = (count & 0x00ff00ff00ff00ff) + ((count >> 8) & 0x00ff00ff00ff00ff);
    count = (count & 0x0000ffff0000ffff) + ((count >> 16) & 0x0000ffff0000ffff);
    return (int)((count & 0x00000000ffffffff) + ((count >> 32) & 0x00000000ffffffff));
}

int MSB64bit(unsigned long long int v) {
    if (v == 0)
    {
        printf("no bit.\n");
        abort();
    }
    v |= (v >> 1);
    v |= (v >> 2);
    v |= (v >> 4);
    v |= (v >> 8);
    v |= (v >> 16);
    v |= (v >> 32);
    return count64bit(v) - 1;
}

mp_limb_t mpz_log2_si(signed long long int op)
{
    mpz_t temp;
    mp_size_t siz;
    int last_bit;
    mpz_init(temp);
    mpz_set_si(temp, op);
    siz = temp->_mp_size > 0 ? temp->_mp_size : -temp->_mp_size;
    last_bit = MSB(temp->_mp_d[siz - 1]);
    last_bit += sizeof(mp_limb_t) * 8 * (siz - 1);
    mpz_clear(temp);
    return last_bit;
}

mp_limb_t mpz_log2_ui(unsigned long long int op)
{
    mpz_t temp;
    mp_size_t siz;
    int last_bit;
    mpz_init(temp);
    mpz_set_si(temp, op);
    siz = temp->_mp_size > 0 ? temp->_mp_size : -temp->_mp_size;
    last_bit = MSB(temp->_mp_d[siz - 1]);
    last_bit += sizeof(mp_limb_t) * 8 * (siz - 1);
    mpz_clear(temp);
    return last_bit;
}

mp_limb_t mpz_log2(mpz_srcptr op)
{
    mp_size_t siz;
    int last_bit;
    siz = op->_mp_size > 0 ? op->_mp_size : -op->_mp_size;
    last_bit = MSB(op->_mp_d[siz - 1]);
    last_bit += sizeof(mp_limb_t) * 8 * (siz - 1);
    return last_bit;
}

mp_limb_t mpz_log2rem(mpz_srcptr op, mpz_ptr rem)
{
    mp_size_t siz;
    int last_bit;
    siz = op->_mp_size > 0 ? op->_mp_size : -op->_mp_size;
    last_bit = MSB(op->_mp_d[siz - 1]);
    last_bit += sizeof(mp_limb_t) * 8 * (siz - 1);
    mpz_set(rem, op);
    mpz_clrbit(rem, last_bit);
    return last_bit;
}
#endif

// Test Program
/*int main()
{
    mpz_t temp1, temp2;
    int i, j;
    mp_limb_t temp;
    mpz_init(temp1);
    mpz_init(temp2);
    for(i = 0;i < 100;i++)
    {
        mpz_set_si(temp2, 1);
        for(j = 0;j < i;j++)
        {
            mpz_mul_ui(temp2, temp2, 2);
        }
        gmp_printf("%d %d\n", i, mpz_log2(temp2));
    }
    for(i = 1;i <= 100;i++)
    {
        mpz_set_si(temp1, i);
        temp = mpz_log2rem(temp1, temp2);
        gmp_printf("%d %d %Zd\n", i, temp, temp2);
    }
    return 0;
}*/
