#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <gmp.h>

#define RAND() ((float)rand()/RAND_MAX)

mpz_t* generate(int n, int rn, int *realsolves, int in, int **imaginalsolves)
{
    mpz_t *ret, *ret2, *swap;
    mpz_t temp, temp2;
    int i, j, istartIndex = 0;
    
    ret = (mpz_t * )malloc(sizeof(mpz_t) * (n + 1));
    ret2 = (mpz_t * )malloc(sizeof(mpz_t) * (n + 1));
    
    mpz_init(temp);
    mpz_init(temp2);
    for(i = 0;i < n + 1;i++)
    {
        mpz_init(ret[i]);
        mpz_init(ret2[i]);
    }
    
    if(rn > 0)
    {
        mpz_set_si(ret[1], 1);
        mpz_set_si(ret[0], -realsolves[0]);
        
        for(i = 1;i < rn;i++)
        {
            mpz_set_si(ret2[0], 0);
            for(j = 0;j < i + 1;j++)
            {
                mpz_set(ret2[j+1], ret[j]);
            }
            
            for(j = 0;j < i + 1;j++)
            {
                mpz_mul_si(temp, ret[j], -realsolves[i]);
                mpz_add(ret2[j], ret2[j], temp);
            }
            
            swap = ret;
            ret = ret2;
            ret2 = swap;
        }
    }else{
        if(in > 0)
        {
            mpz_set_si(ret[2], 1);
            mpz_set_si(temp, imaginalsolves[0][0]);
            mpz_mul_si(ret[1], temp, -2);
            mpz_set_si(temp, imaginalsolves[0][0]);
            mpz_mul_si(temp, temp, imaginalsolves[0][0]);
            mpz_set_si(temp2, imaginalsolves[0][1]);
            mpz_mul_si(temp2, temp2, imaginalsolves[0][1]);
            mpz_add(ret[0], temp, temp2);
            istartIndex = 1;
        }
    }
        
    for(i = istartIndex; i < in;i++)
    {
        mpz_set_si(ret2[0], 0);
        mpz_set_si(ret2[1], 0);
        for(j = 0;j < 2 * i + 1 + rn;j++)
        {
            mpz_set(ret2[j + 2], ret[j]);
        }
        
        for(j = 0;j < 2 * i + 1 + rn;j++)
        {
            mpz_set_si(temp, imaginalsolves[i][0]);
            mpz_mul_si(temp, temp, -2);
            mpz_mul(temp, temp, ret[j]);
            mpz_add(ret2[j + 1], ret2[j + 1], temp);
        }
        
        for(j = 0;j < 2 * i + 1 + rn;j++)
        {
            mpz_set_si(temp, imaginalsolves[i][0]);
            mpz_mul_si(temp, temp, imaginalsolves[i][0]);
            mpz_set_si(temp2, imaginalsolves[i][1]);
            mpz_mul_si(temp2, temp2, imaginalsolves[i][1]);
            mpz_add(temp, temp, temp2);
            mpz_mul(temp, temp, ret[j]);
            mpz_add(ret2[j], ret2[j], temp);
        }
        
        swap = ret;
        ret = ret2;
        ret2 = swap;
    }
    
    mpz_clear(temp);
    mpz_clear(temp2);
    for(i = 0;i < n + 1;i++)
    {
        mpz_clear(ret2[i]);
    }
    
    return ret;
}

void createRandomSolves(int rn, int* realsolves, int in, int** imaginalsolves)
{
    int i, j;
    srand((unsigned int)time(NULL));
    
    for(i = 0;i < rn;i++)
    {
        while(1)
        {
            int found = 0;
            realsolves[i] = rand() * (rand() % 2 == 0 ? 1 : -1);
            for(j = 0;j < i;j++)
            {
                if(realsolves[i] == realsolves[j])
                {
                    found = 1;
                    break;
                }
            }
            
            if(!found)
            {
                break;
            }
        }
        
        printf("%d\n", realsolves[i]);
    }
    
    for(i = 0;i < in;i++)
    {
        while(1)
        {
            int found = 0;
            imaginalsolves[i] = (int * )malloc(sizeof(int) * 2);
            imaginalsolves[i][0] = rand() * (rand() % 2 == 0 ? 1 : -1);
            imaginalsolves[i][1] = rand();
            while(imaginalsolves[i][1] == 0)
            {
                imaginalsolves[i][1] = rand();
            }

            for(j = 0;j < i;j++)
            {
                if(imaginalsolves[i][0] == imaginalsolves[j][0] && imaginalsolves[i][1] == imaginalsolves[j][1])
                {
                    found = 1;
                    break;
                }
            }
            
            if(!found)
            {
                break;
            }
        }
        printf("%d+%di\n", imaginalsolves[i][0], imaginalsolves[i][1]);
    }
}

int main(int argc, char* argv[])
{    
    int * realsolves, **imaginalsolves, i, n, rn, in;
    mpz_t * random_poly;
    FILE * file;
    
    if(argc != 4)
    {
        printf("Usage random_poly_gen.exe <dim> <num of real root> <out>\n");
        return 0;
    }
    
    file = fopen(argv[3], "w");
    
    n = atoi(argv[1]);
    rn = atoi(argv[2]);
    in = (n - rn) / 2;
    
    if((n - rn) % 2 != 0)
    {
        printf("Invalid count");
        return 0;
    }
    
    realsolves = (int * )malloc(sizeof(int) * rn);
    imaginalsolves = (int**)malloc(sizeof(int * ) * in);
    createRandomSolves(rn, realsolves, in, imaginalsolves);
    
    random_poly = generate(n, rn, realsolves, in, imaginalsolves);
    
    for(i = n;i >= 0;i--)
    {
        fprintf(file, "(");
        mpz_out_str(file, 10, random_poly[i]);
        if(i == 0)
        {
            fprintf(file, ")");
        }else{
            fprintf(file, ")*x^%d+", i);
        }
    }
    
    fclose(file);

    return 0;
}
