#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <gmp.h>

#define RAND() ((double)rand()/RAND_MAX)
double SCALE = 10000;

mpz_t* generate(int n, int rn, mpz_t **realsolves, int in, mpz_t **imaginalsolves)
{
    mpz_t *ret, *ret2, *swap;
    mpz_t temp, temp2;
    int i, j, istartIndex = 0;
    
    ret = (mpz_t * )malloc(sizeof(mpz_t) * (n + 1));
    ret2 = (mpz_t * )malloc(sizeof(mpz_t) * (n + 1));
    
    mpz_init(temp);
    mpz_init(temp2);
    for(i = 0;i < n + 1;i++)
    {
        mpz_init(ret[i]);
        mpz_init(ret2[i]);
    }
    
    if(rn > 0)
    {
        mpz_set(ret[1], realsolves[0][1]);
        mpz_set(ret[0], realsolves[0][0]);
        
        for(i = 1;i < rn;i++)
        {
            mpz_set_si(ret2[0], 0);
            for(j = 0;j < i + 1;j++)
            {
                mpz_mul(ret2[j + 1], ret[j], realsolves[i][1]);
            }
            
            for(j = 0;j < i + 1;j++)
            {
                mpz_mul(temp, ret[j], realsolves[i][0]);
                mpz_add(ret2[j], ret2[j], temp);
            }
            
            swap = ret;
            ret = ret2;
            ret2 = swap;
        }
    }else{
        if(in > 0)
        {
            mpz_set(ret[2], imaginalsolves[0][2]);
            mpz_set(ret[1], imaginalsolves[0][1]);
            mpz_set(ret[0], imaginalsolves[0][0]);
            istartIndex = 1;
        }
    }
        
    for(i = istartIndex; i < in;i++)
    {
        mpz_set_si(ret2[0], 0);
        mpz_set_si(ret2[1], 0);
        for(j = 0;j < 2 * i + 1 + rn;j++)
        {
            mpz_mul(ret2[j + 2], ret[j], imaginalsolves[i][2]);
        }
        
        for(j = 0;j < 2 * i + 1 + rn;j++)
        {
            mpz_mul(temp, ret[j], imaginalsolves[i][1]);
            mpz_add(ret2[j + 1], ret2[j + 1], temp);
        }
        
        for(j = 0;j < 2 * i + 1 + rn;j++)
        {
            mpz_mul(temp, ret[j], imaginalsolves[i][0]);
            mpz_add(ret2[j], ret2[j], temp);
        }
        
        swap = ret;
        ret = ret2;
        ret2 = swap;
    }
    
    mpz_clear(temp);
    mpz_clear(temp2);
    for(i = 0;i < n + 1;i++)
    {
        mpz_clear(ret2[i]);
    }
    
    return ret;
}

void createRandomSolves(int rn, mpz_t** realsolves, int in, mpz_t** imaginalsolves)
{
    int i, j;
    mpz_t ztemp, ztemp2;
    mpq_t qtemp, qtemp2;
    double * array, **array2;
    mpz_init(ztemp);
    mpz_init(ztemp2);
    mpq_init(qtemp);
    mpq_init(qtemp2);
    srand((unsigned int)time(NULL));
    
    array = (double * )malloc(sizeof(double) * rn);
    
    for(i = 0;i < rn;i++)
    {
        while(1)
        {
            int found = 0;
            array[i] = RAND() * SCALE * (rand() % 2 == 0 ? 1 : -1);
            for(j = 0;j < i;j++)
            {
                if(array[i] == array[j])
                {
                    found = 1;
                    break;
                }
            }
            
            if(!found)
            {
                mpq_set_d(qtemp, array[i]);
                mpq_get_num(ztemp, qtemp);
                mpz_set(realsolves[i][0], ztemp);
                mpz_neg(realsolves[i][0], realsolves[i][0]);
                mpq_get_den(ztemp, qtemp);
                mpz_set(realsolves[i][1], ztemp);
                printf("%f\n", array[i]);
                break;
            }
        }
    }
    
    free(array);
    array2 = (double**)calloc(in,sizeof(double*));
    
    for(i = 0;i < in;i++)
    {
        while(1)
        {
            int found = 0;
            
            if(array2[i] == NULL)
            {
                array2[i] = (double*)malloc(sizeof(double) * 2);
            }
            
            array2[i][0] = RAND() * SCALE * (rand() % 2 == 0 ? 1 : -1);
            array2[i][1] = RAND() * SCALE;
            
            while(array2[i][1] == 0)
            {
                array2[i][1] = RAND() * rand();
            }

            for(j = 0;j < i;j++)
            {
                if(array2[i][0] == array2[j][0] && array2[i][1] == array2[j][1])
                {
                    found = 1;
                    break;
                }
            }
            
            if(!found)
            {
                mpq_set_d(qtemp, array2[i][0]);
                mpq_set_d(qtemp2, array2[i][1]);
                
                // p^2*q^2
                mpq_get_den(ztemp, qtemp);
                mpq_get_den(ztemp2, qtemp2);
                mpz_pow_ui(imaginalsolves[i][2], ztemp, 2);
                mpz_pow_ui(ztemp, ztemp2, 2);
                mpz_mul(imaginalsolves[i][2], imaginalsolves[i][2], ztemp);
                
                // -2p*r^2*q
                mpz_set_si(imaginalsolves[i][1], -2);
                mpq_get_den(ztemp, qtemp);
                mpq_get_den(ztemp2, qtemp2);
                mpz_mul(imaginalsolves[i][1], imaginalsolves[i][1], ztemp);
                mpz_mul(imaginalsolves[i][1], imaginalsolves[i][1], ztemp2);
                mpz_mul(imaginalsolves[i][1], imaginalsolves[i][1], ztemp2);
                mpq_get_num(ztemp, qtemp);
                mpz_mul(imaginalsolves[i][1], imaginalsolves[i][1], ztemp);
                
                // r^2*q^2+p^2*s^2
                mpq_get_den(ztemp2, qtemp2);
                mpq_get_num(ztemp, qtemp);
                mpz_set(imaginalsolves[i][0], ztemp);
                mpz_mul(imaginalsolves[i][0], imaginalsolves[i][0], ztemp);
                mpz_mul(imaginalsolves[i][0], imaginalsolves[i][0], ztemp2);
                mpz_mul(imaginalsolves[i][0], imaginalsolves[i][0], ztemp2);
                mpq_get_den(ztemp, qtemp);
                mpq_get_num(ztemp2, qtemp2);
                mpz_mul(ztemp, ztemp, ztemp);
                mpz_mul(ztemp2, ztemp2, ztemp2);
                mpz_mul(ztemp, ztemp, ztemp2);
                mpz_add(imaginalsolves[i][0], imaginalsolves[i][0], ztemp);
                printf("%f+i%f\n", array2[i][0], array2[i][1]);
                break;
            }
        }
    }
    
    printf("\n");
    
    for(i = 0;i < in;i++)
    {
        free(array2[i]);
    }
    free(array2);
    
    mpz_clear(ztemp);
    mpz_clear(ztemp2);
    mpq_clear(qtemp);
    mpq_clear(qtemp2);
}

int main(int argc, char* argv[])
{    
    mpz_t **realsolves, **imaginalsolves;
    int i, n, rn, in;
    mpz_t * random_poly;
    FILE * file;
    
    if(argc != 5)
    {
        printf("Usage random_poly_gen.exe <dim> <num of real root> <scale> <out>\n");
        return 0;
    }
    
    file = fopen(argv[4], "w");
    
    n = atoi(argv[1]);
    rn = atoi(argv[2]);
    SCALE = atof(argv[3]);
    in = (n - rn) / 2;
    
    if((n - rn) % 2 != 0)
    {
        printf("Invalid count");
        return 0;
    }
    
    realsolves = (mpz_t**)malloc(sizeof(mpz_t*) * rn);
    for(i = 0;i < rn;i++)
    {
        realsolves[i] = (mpz_t * )malloc(sizeof(mpz_t) * 2);
        mpz_init(realsolves[i][0]);
        mpz_init(realsolves[i][1]);
    }
    imaginalsolves = (mpz_t**)malloc(sizeof(mpz_t * ) * in);
    for(i = 0;i < in;i++)
    {
        imaginalsolves[i] = (mpz_t * )malloc(sizeof(mpz_t) * 3);
        mpz_init(imaginalsolves[i][0]);
        mpz_init(imaginalsolves[i][1]);
        mpz_init(imaginalsolves[i][2]);
    }
    createRandomSolves(rn, realsolves, in, imaginalsolves);
    random_poly = generate(n, rn, realsolves, in, imaginalsolves);
    
    for(i = n;i >= 0;i--)
    {
        fprintf(file, "(");
        mpz_out_str(file, 10, random_poly[i]);
        if(i == 0)
        {
            fprintf(file, ")");
        }else{
            fprintf(file, ")*x^%d+", i);
        }
    }
    
    fclose(file);

    return 0;
}
