#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<math.h>
#include<float.h>
#include<gmp.h>
#include <sys/types.h>
#include <unistd.h>
#include <limits.h>
#include <omp.h>
/*
#include <sys/times.h>
*/
#include <time.h>
#include "lbtype.c"

/* definition */
#define MAXlett (INT_MAX/100)

#define alpha0 1

lbtype_t lbType = LBTYPE_NONE;

// (ax+b)/(cx+d)
struct node{
  mpz_t a;
  mpz_t b;
  mpz_t c;
  mpz_t d;
  int n;
  mpz_t *p;
};

typedef struct node NODE;

struct ans{
  mpq_t p;
  mpq_t q;
};

typedef struct ans ANS;

#ifdef MM
#include "mpz_pool_omp.c"
mpz_pool_t pool;
#define mm_mpz_get(val) mpz_pool_get(pool, (val))
#define mm_mpz_release(val) mpz_pool_release(pool, (val))
#else
#define mm_mpz_get(val) mpz_init((val))
#define mm_mpz_release(val) mpz_clear((val))
#endif

ANS *ap,*az;
omp_lock_t ans_lock;

#include "upperbound.c"
#include "lowerboundCQ.c"
#include "lowerboundKQ.c"
#include "lowerboundLMQ.c"
#include "lowerboundLM.c"
#include "lowerboundLM2.c"
#include "lowerboundFL.c"
#include "lowerboundOFL1.c"
#include "lowerboundOFL2.c"
#include "lowerboundOFL3.c"
#include "lowerboundOFL4.c"
#include "lowerboundOFL5.c"
#include "lowerboundOFL6.c"
#include "lowerboundN.c"
#include "lowerboundCK.c"

int updatePolyInfo(int n, mpz_t *poly, int *sgnv, int *negCoeffCount)
{
    int k, j, sgn, sgn0;
    sgn = mpz_sgn(poly[0]);
    // record negative coefficient
    k=0;
    for(j=0;j<=n;j++)
    {
        if( (sgn0=mpz_sgn(poly[j]))!=0 && sgn0!=sgn )
        {
            sgnv[k]=j;
            k=k+1;
        }
    }
    *negCoeffCount = k;
    return sgn;
}

// calc decartes sign rule
int descartes(int n,mpz_t *poly){

  int i,sign,sign0,change;

  change=0;

  for(i=0;i<=n;i++){
    sign=mpz_sgn(poly[i]);
    if(sign!=0)
      break;
  }

  for(i++;i<=n;i++){
    if( (sign0=mpz_sgn(poly[i]))!=0 && sign0!=sign){
      change++;
      sign=sign0;
    }
  }

  return change;
}

// free NODE
void free_node(NODE *sp){
  int i;
  NODE *tp;
  
  tp = sp;
  mm_mpz_release(tp->a);
  mm_mpz_release(tp->b);
  mm_mpz_release(tp->c);
  mm_mpz_release(tp->d);

  if(tp->p != NULL){
      for(i = 0;i <= tp->n;i++)
      {
          mm_mpz_release(tp->p[i]);
      }
  }
  free(tp);
}

// x->x*scale
void polytrans1(int n,mpz_t *poly,mpz_t scale){

  int i;
  mpz_t tmp2;
  
  mm_mpz_get(tmp2);
  
  mpz_set_ui(tmp2,1);

  for(i=1;i<=n;i++){
      mpz_mul(tmp2, tmp2, scale);
      mpz_mul(poly[i], poly[i], tmp2);
  }

  mm_mpz_release(tmp2);
}

// x->x+shift transform
void polytrans2(int n,mpz_t *poly0,mpz_t *poly,mpz_t shift){

  int i,j;

  for(i=0;i<=n;i++){
    mpz_set(poly0[i],poly[i]);
  }

  for(i=0;i<n;i++){
    for(j=n-1;j>=i;j--){
        mpz_addmul(poly0[j], poly0[j + 1], shift);
    }
  }

}

// x->1/(x+1)
void polytrans3(int n,mpz_t *poly0,mpz_t *poly){

  int i,j;

  for(i=0;i<=n;i++){
    mpz_set(poly0[n-i],poly[i]);
  }

  for(i=0;i<n;i++){
    for(j=n-1;j>=i;j--){
      mpz_add(poly0[j],poly0[j],poly0[j+1]);
    }
  }

}

// x->x+1
void polytrans4(int n,mpz_t *poly0,mpz_t *poly){

  int i,j;

  for(i=0;i<=n;i++){
    mpz_set(poly0[i],poly[i]);
  }

  for(i=0;i<n;i++){
    for(j=n-1;j>=i;j--){
      mpz_add(poly0[j],poly0[j],poly0[j+1]);
    }
  }

}

// x->x+1
void polytrans5(int n,mpz_t *poly){

  int i,j;

  for(i=0;i<n;i++){
    for(j=n-1;j>=i;j--){
      mpz_add(poly[j],poly[j],poly[j+1]);
    }
  }

}

// x->x+shift
void polytrans6(int n,mpz_t *poly,mpz_t shift){

  int i,j;

  for(i=0;i<n;i++){
    for(j=n-1;j>=i;j--){
        mpz_addmul(poly[j], poly[j + 1], shift);
    }
  }

}

// x=>-x
void polytrans7(int n,mpz_t *poly2,mpz_t *poly){

  int i,val;

  mpz_set(poly2[0],poly[0]);

  val=0;

  for(i=1;i<=n;i++){
    if(val==0){
      mpz_neg(poly2[i],poly[i]);
    }
    else{
      mpz_set(poly2[i],poly[i]);
    }
    val=1-val;
  }

}

// poly->poly/x
void polytranss(int n,mpz_t *poly){

  int i;

  for(i=0;i<n;i++){
    mpz_set(poly[i],poly[i+1]);
  }

}

int ap0,az0;

// write answer x=0 (b/d)
int ans1(NODE *wp,int sw){

  omp_set_lock(&ans_lock);
  az0++;
  if( (az=(ANS *)realloc(az,(az0)*sizeof(ANS)))==NULL ){
    return 0;
  }
  
  mpq_init(az[az0-1].p);
  mpq_init(az[az0-1].q);

  mpq_set_num(az[az0-1].p,wp->b);
  mpq_set_den(az[az0-1].p,wp->d);
  
  if(sw==2){
    mpq_neg(az[az0-1].p,az[az0-1].p);
  }

  mpq_set(az[az0-1].q,az[az0-1].p);
  omp_unset_lock(&ans_lock);
  return 1;
}

// write answer
int ans2(NODE *wp, int* sgnv, int sw){

  mpz_t tmp1;
  mm_mpz_get(tmp1);

  omp_set_lock(&ans_lock);
  ap0++;
  if( (ap=(ANS *)realloc(ap,(ap0)*sizeof(ANS)))==NULL ){
    return 0;
  }
  
  // [p,q] is answer
  mpq_init(ap[ap0-1].p);
  mpq_init(ap[ap0-1].q);

  // p is x=0 so b/d
  mpq_set_num(ap[ap0-1].p,wp->b);
  mpq_set_den(ap[ap0-1].p,wp->d);

  if(mpz_sgn(wp->c) != 0){
    // when c is not 0 answer is x=infty so a/c
    mpq_set_num(ap[ap0-1].q,wp->a);
    mpq_set_den(ap[ap0-1].q,wp->c);
  }
  else{
    // get upperbound
    // q is (a*ub+b)/(c*ub+d)
    upperbound(wp->n, wp->p, tmp1, sgnv);
 
    mpz_addmul(wp->b,wp->a,tmp1);
    mpz_addmul(wp->d,wp->c,tmp1);

    mpq_set_num(ap[ap0-1].q,wp->b);
    mpq_set_den(ap[ap0-1].q,wp->d);
  }

  // when [-infty,0], reverse p and q's sign
  if(sw==2){
    mpq_neg(ap[ap0-1].q,ap[ap0-1].q);
    mpq_neg(ap[ap0-1].p,ap[ap0-1].p);
  }

  // set as p<q
  if(mpq_cmp(ap[ap0-1].p,ap[ap0-1].q)>0){
    mpq_swap(ap[ap0-1].p,ap[ap0-1].q);
  }
  omp_unset_lock(&ans_lock);

  mm_mpz_release(tmp1);
  return 1;
}

int uspensky_impl(NODE* node, int sw,int**clLength,mpz_t** cl, int**sgnv, int** useCount)
{
    int i, r, s, s1, s2, flag, sgn, nc;
    NODE *tp, *up;
    mpz_t *poly1, *poly2, tmp1, tmp6;
    int n1, n2 = 0;
    int tid;
    int *tsgnv, *tuseCount,*tclLength;
    mpz_t * tcl;
    tid = omp_get_thread_num();
    tsgnv = sgnv[tid];
    tuseCount = useCount[tid];
    tclLength = clLength[tid];
    tcl = cl[tid];

    mm_mpz_get(tmp1);
    mm_mpz_get(tmp6);
    // get lowerbound
    mpz_set_ui(tmp1, 0);
    sgn = updatePolyInfo(node->n, node->p, tsgnv, &nc);
    flag = 0;
    if((lbType & LBTYPE_CKN) == LBTYPE_CKN)
    {
        flag = lowerboundCK(node->n, node->p, tmp1, tmp6, sgn, nc, tsgnv);
    }
    if((lbType & LBTYPE_CQ) == LBTYPE_CQ)
    {
        lowerboundCQ(node->n, node->p, tmp1, sgn, nc, tsgnv);
    }
    if((lbType & LBTYPE_KQ) == LBTYPE_KQ)
    {
        lowerboundKQ(node->n, node->p, tmp1, sgn, nc, tsgnv);
    }
    if((lbType & LBTYPE_LMQ) == LBTYPE_LMQ)
    {
        lowerboundLMQ(node->n, node->p, tmp1, sgn, nc, tsgnv);
    }
    if((lbType & LBTYPE_FL) == LBTYPE_FL)
    {
        lowerboundFL(node->n, node->p, tmp1, sgn, nc, tclLength, tcl);
    }
    if((lbType & LBTYPE_OFL1) == LBTYPE_OFL1)
    {
        lowerboundOFL1(node->n, node->p, tmp1, sgn, nc, tclLength, tcl);
    }
    if((lbType & LBTYPE_OFL2) == LBTYPE_OFL2)
    {
        lowerboundOFL2(node->n, node->p, tmp1, sgn, nc, tclLength, tcl);
    }
    if((lbType & LBTYPE_OFL3) == LBTYPE_OFL3)
    {
        lowerboundOFL3(node->n, node->p, tmp1, sgn, nc, tclLength, tcl);
    }
    if((lbType & LBTYPE_OFL4) == LBTYPE_OFL4)
    {
        lowerboundOFL4(node->n, node->p, tmp1, sgn, nc, tclLength, tcl);
    }
    if((lbType & LBTYPE_LM) == LBTYPE_LM)
    {
        lowerboundLM(node->n, node->p, tmp1, sgn, nc, tsgnv);
    }
    if((lbType & LBTYPE_LM2) == LBTYPE_LM2)
    {
        lowerboundLM2(node->n, node->p, tmp1, sgn, nc, tsgnv, tuseCount);
    }
    if((lbType & LBTYPE_OFL5) == LBTYPE_OFL5)
    {
        lowerboundOFL5(node->n, node->p, tmp1, sgn, nc);
    }
    if((lbType & LBTYPE_OFL6) == LBTYPE_OFL6)
    {
        lowerboundOFL6(node->n, node->p, tmp1, sgn, nc);
    }
    if((lbType & LBTYPE_CKN) == LBTYPE_CKN && flag)
    {
        lowerboundN(node->n, node->p, tmp1, tmp6, sgn);
    }
    if((lbType & LBTYPE_N) == LBTYPE_N)
    {
        mpz_neg(tmp6, (node->p)[0]);
        if(tsgnv[0]==1)
        {
	    mpz_fdiv_q(tmp6, tmp6, node->p[tsgnv[0]]);
            lowerboundN(node->n, node->p, tmp1, tmp6, sgn);
        }
    }

    if(mpz_cmp_si(tmp1,alpha0)>0)
    {
        // x->x+tmp1
        /*polytrans6(wp->n, wp->p, tmp1);
        mpz_addmul(wp->b, wp->a, tmp1);
        mpz_addmul(wp->d, wp->c, tmp1);*/
        // x->x*scale
        polytrans1(node->n, node->p, tmp1);
        mpz_mul(node->a, node->a, tmp1);
        mpz_mul(node->c, node->c, tmp1);
        
        /*polytrans5(wp->n,wp->p);
        
        mpz_add(wp->b,wp->b,wp->a);
        mpz_add(wp->d,wp->d,wp->c);
        
        // check a[0] is 0
        if(mpz_sgn((wp->p)[0]) == 0)
        {
        
        // write answer 0
        if(ans1(wp, sw) == 0)
        {
        // when fail alloc
        return 0;
        }
        
        // decrease dim
        polytranss(wp->n,wp->p);
        (wp->n)--;
        }*/
        
    }
    
    // calc decartes sign rule
    s = descartes(node->n, node->p);
    
    // when no change=> no real root
    if(s == 0)
    {
        // free node
      free_node(node);
      mm_mpz_release(tmp1);
      mm_mpz_release(tmp6);
      return 1;
    }
    
    // when only 1 root
    if(s==1)
    {
        
        // get upperbound and write answer
        if(ans2(node, tsgnv, sw) == 0)
        {
            return 0;
        }
        
        // free node
	free_node(node);
	mm_mpz_release(tmp1);
	mm_mpz_release(tmp6);
	return 1;
    }
    
    n1 = node->n;
    if((poly1=(mpz_t*)malloc((n1+1)*sizeof(mpz_t))) == NULL)
    {
        return 0;
    }
    
    for(i = 0;i <= node->n;i++)
    {
        mm_mpz_get(poly1[i]);
        mpz_set_si(poly1[i], 0);
    }
    
    r = 0;
    
    // x->x+1
    polytrans4(n1, poly1, node->p);
    
    tp = NULL;
    
    // check a[0] is 0
    if(mpz_sgn(poly1[0])==0)
    {
        
        if((tp=(NODE*)malloc(sizeof(NODE))) == NULL)
        {
            return 0;
        }
        
        // copy and x->x+1
        mm_mpz_get(tp->a);
        mm_mpz_get(tp->b);
        mm_mpz_get(tp->c);
        mm_mpz_get(tp->d);
        mpz_set(tp->a, node->a);
        mpz_add(tp->b, node->a, node->b);
        mpz_set(tp->c, node->c);
        mpz_add(tp->d, node->c, node->d);
        
        // write answer 0
        if(ans1(tp, sw) == 0)
        {
            return 0;
        }
        
        // decrease dim
        polytranss(n1, poly1);
        n1--;
        
        r = 1;
    }
    
    // calc decartes sign rule where [1,infty]
    s1 = descartes(n1, poly1);
    
    // calc decartes sign rule where [0,1]
    s2 = s - s1 - r;
    
    poly2 = NULL;
    
    if(s2 > 1)
    {
        n2 = node->n;
        if((poly2=(mpz_t*)malloc((n2+1)*sizeof(mpz_t))) == NULL)
        {
            return 0;
        }
        
        for(i = 0;i <= n2;i++)
        {
            mm_mpz_get(poly2[i]);
            mpz_set_si(poly2[i], 0);
        }
        
        // x->1/(x+1)
        polytrans3(n2, poly2, node->p);
        
        // check a[0] is 0
        if(mpz_sgn(poly2[0])==0)
        {
            // decrease dim
            polytranss(n2, poly2);
            n2--;
        }
        
        // calc decartes sign rule
        s2 = descartes(n2, poly2);
    }
    
    if(s1 == 0)
    {
        // when no real root
        if(tp != NULL)
        {
            free_node(tp);
        }
	
	if(s2 == 0)
	  {
	    // when no real root
	    free_node(node);
	    mm_mpz_release(tmp1);
	    mm_mpz_release(tmp6);
	    return 1;
	  }
	
	if((up=(NODE *)malloc(sizeof(NODE))) == NULL)
	  {
	    return 0;
	  }
	
	// copy and x->1/(x+1)
	mm_mpz_get(up->a);
	mm_mpz_get(up->b);
	mm_mpz_get(up->c);
	mm_mpz_get(up->d);
	mpz_set(up->a, node->b);
	mpz_add(up->b, node->a, node->b);
	mpz_set(up->c, node->d);
	mpz_add(up->d, node->c, node->d);
	
	up->p = poly2;
	up->n = n2;
	
	if(s2 == 1)
	  {
	    // when only one root
	    if(ans2(up, tsgnv, sw)==0)
	      {
		return 0;
	      }
	    free_node(up);
	  }
	else
	  {
	    uspensky_impl(up, sw, clLength, cl, sgnv, useCount);
	  }

	free_node(node);
	mm_mpz_release(tmp1);
	mm_mpz_release(tmp6);
	return 1;
    }
    
    if(tp == NULL)
    {
        
        if((tp=(NODE *)malloc(sizeof(NODE))) == NULL)
        {
            return 0;
        }
        
        // copy and x->x+1
        mm_mpz_get(tp->a);
        mm_mpz_get(tp->b);
        mm_mpz_get(tp->c);
        mm_mpz_get(tp->d);
        mpz_set(tp->a, node->a);
        mpz_add(tp->b, node->a, node->b);
        mpz_set(tp->c, node->c);
        mpz_add(tp->d, node->c, node->d);
    }
    
    tp->p = poly1;
    tp->n = n1;
    
    if(s1 == 1)
    {
        // only one real root
        if(ans2(tp, tsgnv, sw)==0)
        {
            return 0;
        }
        
        free_node(tp);
    }
    else
    {
        #pragma omp task
        uspensky_impl(tp, sw, clLength, cl, sgnv, useCount);
    }
    
    if(s2 == 0)
    {
        // when no real root
        #pragma omp taskwait
	free_node(node);
	mm_mpz_release(tmp1);
	mm_mpz_release(tmp6);
	return 1;
    }
    
    if((up=(NODE *)malloc(sizeof(NODE))) == NULL)
    {
        return 0;
    }
    
    // copy and x->1/(x+1)
    mm_mpz_get(up->a);
    mm_mpz_get(up->b);
    mm_mpz_get(up->c);
    mm_mpz_get(up->d);
    mpz_set(up->a, node->b);
    mpz_add(up->b, node->a, node->b);
    mpz_set(up->c, node->d);
    mpz_add(up->d, node->c, node->d);
    
    up->p = poly2;
    up->n = n2;
    
    if(s2 == 1)
    {
        // when only one root
        if(ans2(up, tsgnv, sw)==0)
        {
            return 0;
        }
        free_node(up);
    }
    else
    {
        #pragma omp task
        uspensky_impl(up, sw, clLength, cl, sgnv, useCount);
    }
    
    #pragma omp taskwait
    free_node(node);
    mm_mpz_release(tmp1);
    mm_mpz_release(tmp6);
    return 1;
}

int uspensky(int n,mpz_t *poly,int sw,int** clLength, mpz_t** cl, int** sgnv, int** useCount)
{
    int i, s;
    NODE *sp;
    mpz_t tmp1, tmp6;
    int tid;
    int * tsgnv;
    tid = omp_get_thread_num();
    tsgnv = sgnv[tid];
    
    mm_mpz_get(tmp1);
    mm_mpz_get(tmp6);
    
    // decartes sign rule
    s = descartes(n, poly);
    
    // if 0 no sign change => no real root
    if(s == 0)
    {
        mm_mpz_release(tmp1);
        mm_mpz_release(tmp6);
        return 1;
    }
    
    // there is only 1 real root
    if(s==1)
    {
        ap0++;
        if( (ap=(ANS*)realloc(ap,(ap0)*sizeof(ANS)))==NULL )
        {
            return 0;
        }
        mpq_init(ap[ap0-1].p);
        mpq_init(ap[ap0-1].q);
        
        // calc upperbound to determine [0,infty] => [0,upperbound]
        upperbound(n, poly, tmp1, tsgnv);
        
        if(sw == 1)
        {
            // when x belongs to [0,infty]
            //mpq_set_si(ap[ap0-1].p,0,1);
            mpq_set_z(ap[ap0 - 1].q, tmp1);
        }
        else if(sw == 2)
        {
            // when x belongs to [-infty,0]
            mpq_set_z(ap[ap0 - 1].p, tmp1);
            mpq_neg(ap[ap0 - 1].p, ap[ap0 - 1].p);
            //mpq_set_si(ap[ap0-1].q,0,1);
        }
        
        mm_mpz_release(tmp1);
        mm_mpz_release(tmp6);
        return 1;
    }
    
    // when decartes sign is more than 2. devide region
    if((sp=(NODE*)malloc(sizeof(NODE))) == NULL)
    {
        mm_mpz_release(tmp1);
        mm_mpz_release(tmp6);
        return 0;
    }
    if((sp->p=(mpz_t*)malloc((n+1)*sizeof(mpz_t))) == NULL)
    {
        mm_mpz_release(tmp1);
        mm_mpz_release(tmp6);
        return 0;
    }
    sp->n = n;
    for(i = 0;i <= n;i++)
    {
        mm_mpz_get(sp->p[i]);
        mpz_set(sp->p[i], poly[i]);
    }
    
    mm_mpz_get(sp->a);
    mm_mpz_get(sp->b);
    mm_mpz_get(sp->c);
    mm_mpz_get(sp->d);
    mpz_set_si(sp->a, 1);
    mpz_set_si(sp->b, 0);
    mpz_set_si(sp->c, 0);
    mpz_set_si(sp->d, 1);
    return uspensky_impl(sp, sw, clLength, cl, sgnv, useCount);
}

int getline2(char *line,FILE *fp){

  char c;
  int i;

  for(i=0;i<MAXlett;i++){
    c=fgetc(fp);
    if (c==EOF || c=='\n'){
      line[i]='\0';
      break;
    }
    line[i]=c;
  }

  if(i==MAXlett)
    return 0;
  else
    return 1;
}

int frzall(int n, mpz_t * poly, mpz_t * poly2){
  int ret = 1, i,j;
  int **sgnv, **useCount;
  int **clLength;
  mpz_t **cl;
  int tCount;
  if(mpz_sgn(poly[0])==0){
    polytranss(n,poly);
    n--;
    az0=1;
    if( (az=(ANS *)realloc(az,(az0)*sizeof(ANS)))==NULL ){
      return(-1);
    }
    mpq_init(az[az0-1].p);
    mpq_init(az[az0-1].q);
  }
  #pragma omp parallel
  {
      #pragma omp single
      {
          tCount = omp_get_num_threads();
          sgnv = (int**)malloc(sizeof(int * )*tCount);
          useCount = (int**)malloc(sizeof(int * )*tCount);
          clLength = (int**)malloc(sizeof(int * )*tCount);
          cl = (mpz_t**)malloc(sizeof(mpz_t * )*tCount);
          for(i = 0;i < tCount;i++)
          {
              sgnv[i] = (int * )malloc(sizeof(int) * (n + 1));
              useCount[i] = (int * )malloc(sizeof(int) * (n + 1));
              clLength[i] = (int * )malloc(sizeof(int) * (n + 1));
              cl[i] = (mpz_t * )malloc(sizeof(mpz_t) * (n + 1));
              for(j = 0;j <= n;j++)
              {
                  mm_mpz_get(cl[i][j]);
              }
          }
          
          printf("Thread Count:%d\n", tCount);
          #pragma omp task
          {
              printf("calc +.\n");
              if(uspensky(n,poly,1,clLength,cl,sgnv,useCount)==0)
              {
                  printf("calc + error end.\n");
                  ret &= 0;
              }
              printf("calc +end.\n");
          }
	  #pragma omp task
          {
              polytrans7(n,poly2,poly);
              printf("calc -.\n");
              if(uspensky(n, poly2, 2,clLength,cl,sgnv,useCount) == 0)
              {
                  printf("calc - error end.\n");
                  ret &= 0;
              }
              printf("calc -end.\n");
          }
	  #pragma omp taskwait
          printf("count:%d\n", ap0 + az0);
      }
  }
  
  for(i = 0;i < tCount;i++)
  {
      free(sgnv[i]);
      free(useCount[i]);
      free(clLength[i]);
      for(j = 0;j <= n;j++)
      {
          mm_mpz_release(cl[i][j]);
      }
      free(cl[i]);
  }
  free(sgnv);
  free(useCount);
  free(clLength);
  free(cl);
  
  return ret;
}

int main(int argc,char *argv[]){

  char *line,*line2;
  FILE *fp;
  mpz_t hc;
  int i, n;
  mpz_t * poly, * poly2;
  double start, end;
  
  #ifdef MM
  mpz_pool_init(pool, 10000);
  #endif
  mm_mpz_get(hc);
  mpz_set_si(hc, 0);

  if (argc<4){
    printf("<poly file> <output> <lbtype>.\n");
    return 0;
  }
  
  fp=fopen(argv[1],"r");

  if ((line=(char *)malloc(MAXlett*sizeof(char)))==NULL){
    printf("out of momory.\n");
    return 0;
  }

  printf("read file.\n");

  if (getline2(line,fp)!=1){
    printf("too many letters in one line.\n");
    return 0;
  }

  n=0;
  line2=strpbrk(line,"(");
  line=line2+1;
  line2=strpbrk(line,")");
  *line2='\0';
  mpz_set_str(hc,line,10);
  line=line2+1;
  if (*line!='\0'){
    line2=strpbrk(line,"^+");
    if(*line2=='^'){
      line=line2+1;
      line2=strpbrk(line,"+");
      *line2='\0';
      n=atoi(line);
    }
    else if(*line2=='+'){
      n=1;
    }
    line=line2+1;
  }
  else{
    printf("degree 0\n");
    return 0;
  }

  printf("read file end.\n");
  
  if((poly=(mpz_t *)malloc((n+1)*sizeof(mpz_t)))==NULL){
    printf("Out of memory.\n");
    return 0;
  }

  if((poly2=(mpz_t *)malloc((n+1)*sizeof(mpz_t)))==NULL){
    printf("Out of memory.\n");
    return 0;
  }

    
  for(i=0;i<=n;i++){
    mpz_init(poly[i]);
    mpz_init(poly2[i]);
  }

  mpz_set(poly[n],hc);

  while(1){
    
    line2=strpbrk(line,"(");
    line=line2+1;
    line2=strpbrk(line,")");
    *line2='\0';
    mpz_set_str(hc,line,10);
    line=line2+1;
    if (*line!='\0'){
      line2=strpbrk(line,"^+");
      if(*line2=='^'){
        line=line2+1;
        line2=strpbrk(line,"+");
        *line2='\0';
        i=atoi(line);
        mpz_set(poly[i],hc);
      }
      else if(*line2=='+'){
        mpz_set(poly[1],hc);
      }
      line=line2+1;
    }
    else{
      mpz_set(poly[0],hc);
      break;
    }
  }

  fclose(fp);
  mm_mpz_release(hc);
  
  lbType = atoi(argv[3]);
  printLowerbound(lbType);

  ap0=0;
  az0=0;
  ap=NULL;
  az = NULL;
  omp_init_lock(&ans_lock);
  start = omp_get_wtime();
  if(frzall(n,poly,poly2)==0)
    return 0;
  
  end = omp_get_wtime();

  fp=fopen(argv[2],"w");

  for(i=0;i<ap0;i++){
    gmp_fprintf(fp,"[%Qd,%Qd]\n",ap[i].p,ap[i].q);
  }

  for(i=0;i<az0;i++){
    gmp_fprintf(fp,"[%Qd,%Qd]\n",az[i].p,az[i].q);
  }

  fclose(fp);
  printf("%.2fsec elapsed\n", end - start);

  return 0;
}
